#####################################################################
##
##  Configuration script to modify keystores for writable support
##
#####################################################################

import sys
import re
import getopt

def modifyKeyStore ( keyStoreName, userName, undo):
    # Search for the keystore and modify the patch and readOnly attributes
    keyStoreList = AdminConfig.list( "KeyStore" )
    keyStoreList = keyStoreList.splitlines()
    
    #set to true if the keystore is successfully modified
    modified = 0

    for keyStore in keyStoreList:
        name = AdminConfig.showAttribute( keyStore, "name" )
        location = AdminConfig.showAttribute( keyStore, "location" )
        type = AdminConfig.showAttribute( keyStore, "type" )
        readOnly = AdminConfig.showAttribute( keyStore, "readOnly" )

        if ((( type == "JCERACFKS" ) 
             or ( type == "JCECCARACFKS" )) 
            and readOnly == "true"):        
            #search = re.compile("^%s$" % keyStoreName).search(name)        
            if keyStoreName == name:
                if( undo == 0 ):
                    modifiedLocation = setKeyringLocation( location, userName )
                else:
                    modifiedLocation = setKeyringLocation( location, "" )
                #endElse
                       
                if modifiedLocation is not None:
                    AdminConfig.modify( keyStore, [["location", modifiedLocation]] )
                    if( undo == 0 ):
                        AdminConfig.modify( keyStore, [["readOnly", "false"]] )
                    else:
                        AdminConfig.modify( keyStore, [["readOnly", "true"]] )
                    #endElse
                    
                    if undo == 0:
                        print "Modified " + keyStoreName + ", location=" + modifiedLocation + " readOnly=false"
                    else:
                        print "Modified " + keyStoreName + ", location=" + modifiedLocation + " readOnly=true"
                    #endElse
                    
                    modified = 1
                    #AdminConfig.save()
                    return
                #endIf
        #endIf
    #endFor 
    if modified == 1:
        print "KeyStore " + keyStoreName + " does not exist"

def checkIfKeyStoreExists( keyStoreName ):
    keyStoreList = AdminConfig.list( "KeyStore" )
    keyStoreList = keyStoreList.splitlines()
    
    for keyStore in keyStoreList:
        name = AdminConfig.showAttribute( keyStore, "name" )
        if keyStoreName == name:
            return "true"
        #endIf
    #endFor
    
    return None
#endDef

def augmentKeyStore ( keyStoreName, userNameCR, userNameSR ):
    # Search for the keystore and modify the patch and readOnly attributes
    keyStoreList = AdminConfig.list( "KeyStore" )
    keyStoreList = keyStoreList.splitlines()

    for keyStore in keyStoreList:
        name = AdminConfig.showAttribute( keyStore, "name" )
        location = AdminConfig.showAttribute( keyStore, "location" )
        type = AdminConfig.showAttribute( keyStore, "type" )
        provider = AdminConfig.showAttribute( keyStore, "provider" )
        managementScope = AdminConfig.showAttribute( keyStore, "managementScope" )
        password = AdminConfig.showAttribute( keyStore, "password" )
           
        if ( type == "JCERACFKS" ) or ( type == "JCECCARACFKS" ):        
            #search = re.compile("^%s$" % keyStoreName).search(name)        
            if keyStoreName == name:
                #if(undo == 0):
                modifiedLocationCR = setKeyringLocation( location, userNameCR )
                modifiedLocationSR = setKeyringLocation( location, userNameSR )
                #endElse
                       
                if modifiedLocationCR is not None and modifiedLocationSR is not None:
                    #if(undo == 0):
                    keyStoreNameSR = name + "-SR"
                    keyStoreNameCR = name + "-CR"
                        
                    security = AdminConfig.getid("\"/Security:/\"")
                        
                    if checkIfKeyStoreExists( keyStoreNameSR ) is None:
                       props = ( [['name', keyStoreNameCR], 
                                ['password', "password"], 
                                ['provider', provider], 
                                ['location', modifiedLocationCR], 
                                ['type', type], 
                                ['fileBased', 'false'], 
                                ['hostList', ''], 
                                ['description', 'Writable control region keyring'], 
                                ['usage', keyStoreName], #set this to the parent keystore
                                ['managementScope', managementScope]] )
                       AdminConfig.create( "KeyStore", security, props )
                       print "Created \"" + keyStoreNameCR + "\", location=" + modifiedLocationCR + " readOnly=false"
                    #endIf
                        
                    if checkIfKeyStoreExists( keyStoreNameSR ) is None:
                       props = ( [['name', keyStoreNameSR], 
                               ['password', "password"], 
                               ['provider', provider], 
                               ['location', modifiedLocationSR], 
                               ['type', type], 
                               ['fileBased', 'false'], 
                               ['hostList', ''], 
                               ['description', 'Writable servant region keyring'], 
                               ['usage', keyStoreName], #set this to the parent keystore
                               ['managementScope', managementScope]] )
                       AdminConfig.create( "KeyStore", security, props )
                       print "Created \"" + keyStoreNameSR + "\", location=" + modifiedLocationSR + " readOnly=false"
                    #endIf
                    return
                #endIf
        #endIf
    #endFor 
#endDef

def setKeyringLocation ( keyStoreLocation, user ):
    
    if isValidKeyring( keyStoreLocation ) == 1:
        splitKeyring = keyStoreLocation.split( "/" )
        return splitKeyring[0] + "//" + user + "/" + splitKeyring[3]
    else:
        return None
    #endElse
#endDef

def isValidKeyring ( keyStoreLocation ):
    keyringPrefix = "safkeyring://"

    if ( ( not keyStoreLocation.startswith( keyringPrefix ) ) or ( keyStoreLocation.startswith( keyringPrefix ) and keyStoreLocation.endswith( keyringPrefix ) ) or keyStoreLocation.endswith( "/" ) ):
        print "The SAF keyring location specified as " + keyStoreLocation + ", is not a valid URI"
        return 0
    #endIf

    splitKeyring = keyStoreLocation.split( "/" )
    return 1
#endDef

def processKeyStores( userNameCR, userNameSR ):
    sslConfigList = AdminConfig.list( "SSLConfig" )
    sslConfigList = sslConfigList.splitlines()
    
    #process SSL Configs
    for sslConfig in sslConfigList:
        alias = AdminConfig.showAttribute( sslConfig, "alias" )
        if alias.endswith( "DefaultIIOPSSL" ):
            print "Skipping SSLConfig \"" + alias + "\""
            continue
        #endIf
        print "Processing SSLConfig \"" + alias + "\""
        setting = AdminConfig.showAttribute( sslConfig, "setting" )
        keyStoreObject = AdminConfig.showAttribute( setting, "keyStore" )
        keyStoreName = AdminConfig.showAttribute( keyStoreObject, "name" )
        print "Processing Keystore \"" + keyStoreName + "\""
        augmentKeyStore( keyStoreName, userNameCR, userNameSR )
    #endFor
    
    #process remaining default keystores
    # Search for the keystore and modify the patch and readOnly attributes
    keyStoreList = AdminConfig.list( "KeyStore" )
    keyStoreList = keyStoreList.splitlines()

    for keyStore in keyStoreList:
        name = AdminConfig.showAttribute( keyStore, "name" )
        
        if( name.endswith( "RSATokenKeyStore" ) ):
            print "Processing Keystore \"" + name + "\""
            augmentKeyStore( name, userNameCR, userNameSR )
        
        if ( name.endswith( "DefaultRootStore" ) 
            or name.endswith( "DefaultSignersStore" ) 
            or name.endswith( "RSATokenRootStore" )):
            print "Processing Keystore \"" + name + "\""
            modifyKeyStore( name, userNameCR, 0 )
        #endIf
    #endFor
#endDef

def usage():
    print "Usage: enableWritableKeyrings --keyStore=<keyStoreName> --userNameCR=<Controller User> --userNameSR=<Servant User> --all"
    print ""
    print "--keyStore - the keystore to update.  If a specific keystore name is supplied, --all is ignored"
    print "           - If --userNameCR is supplied, and --userNameSR is omitted, then the keystore path is modified to include the CR user"
    print "           - If --userNameCR and --userNameSR are supplied, the keystore is augmented, creating two new keystores each with a path pointing to the CR and SR, respectively."   
    print "--userNameCR - the userid of the control region."
    print "--userNameSR - the userid of the servant region."
    print "--all - processes all keystores in the security configuration.   If this option is supplied, --keyStoreName is ignored."
    print ""
#endDef

def main(): 
    try:
        opts, args = getopt.getopt( sys.argv[0:], "", ["help", "keyStore=", "userNameCR=", "userNameSR=", "all"] )
    except getopt.GetoptError:
        usage()
        sys.exit( 2 )
    userNameCR = None
    userNameSR = None
    keyStore = None
    undo = 0
    
    for o, val in opts:
        if o in ( "-h", "--help" ):
            usage()
            sys.exit()
        if o in ( "-k", "--keyStore" ):
            keyStore = val
        if o in ( "-n", "--userNameCR" ):
            userNameCR = val
        if o in ( "-n", "--userNameSR" ):
            userNameSR = val
        if o in ( "-a", "--all" ):
            all = 1
        #endIf
    #endFor
    
    if keyStore is not None:
        if userNameCR is None:
            usage()
            sys.exit( 2 )
        #endIf
        if userNameSR is None:
            modifyKeyStore(keyStore, userNameCR, undo)
        else:
            augmentKeyStore(keyStore, userNameCR, userNameSR)
        #endElse
    else:
        if ((userNameCR is not None and userNameSR is not None) 
            and all == 1):       
            #process all keystore configurations
            processKeyStores ( userNameCR, userNameSR )
        else:
            usage()
            sys.exit( 2 )
        #endElse
    #endElse
    
    #Save Updates
    AdminConfig.save()
    print "Configuration saved"
#endDef

main()

#endElse 